"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PLUGIN_IGNORE_RESULT = void 0;
const flatten_1 = __importDefault(require("lodash/flatten"));
const domainWhitelist_1 = __importDefault(require("../basePlugins/domainWhitelist"));
const webImageLoader_1 = __importDefault(require("../basePlugins/webImageLoader"));
const flow_1 = require("../utils/flow");
exports.PLUGIN_IGNORE_RESULT = Symbol('PLUGIN_IGNORE_RESULT');
const pluginLoader = (config, logger) => {
    let loadedPlugins = [
        {
            name: 'webImageLoader',
            instance: (0, webImageLoader_1.default)()({ config, PLUGIN_IGNORE_RESULT: exports.PLUGIN_IGNORE_RESULT }),
        },
        {
            name: 'domainWhitelist',
            instance: (0, domainWhitelist_1.default)({ whitelist: config.whitelist })({
                config,
                PLUGIN_IGNORE_RESULT: exports.PLUGIN_IGNORE_RESULT,
            }),
        },
    ];
    if (config.plugins) {
        const { paths, plugins = [] } = config.plugins;
        loadedPlugins = plugins
            .filter((plugin) => !plugin.disabled)
            .reduce((acc, { name, instance, options }) => {
            let plugin;
            if (instance) {
                plugin = instance;
            }
            else {
                // eslint-disable-next-line @typescript-eslint/no-var-requires
                const loaded = require(require.resolve(name, {
                    paths,
                }));
                plugin = loaded(options);
            }
            acc.push({
                name,
                instance: plugin({
                    config,
                    PLUGIN_IGNORE_RESULT: exports.PLUGIN_IGNORE_RESULT,
                }),
            });
            return acc;
        }, loadedPlugins);
    }
    const getFnsFromPlugins = (fn) => loadedPlugins
        .map(({ instance }) => instance[fn])
        .filter((f) => f !== undefined);
    const onPluginError = (e) => {
        logger?.debug(e);
        logger?.debug('Trying next plugin');
    };
    return {
        urlTransform: async (url, req) => {
            const fns = getFnsFromPlugins('urlTransform');
            if (fns.length === 0)
                return url;
            try {
                const result = await (0, flow_1.tryEach)(fns, {
                    onError: onPluginError,
                    ignoreResult: (r) => r === exports.PLUGIN_IGNORE_RESULT,
                })(url, req);
                return result || url;
            }
            catch (e) {
                return url;
            }
        },
        inputImageLoader: async (imageUrl) => {
            const fns = getFnsFromPlugins('inputImageLoader');
            try {
                const result = await (0, flow_1.tryEach)(fns, {
                    onError: onPluginError,
                    ignoreResult: (r) => r === exports.PLUGIN_IGNORE_RESULT,
                })(imageUrl);
                return result;
            }
            catch (e) {
                throw new Error(`[PLUGIN:inputImageLoader] failed. No plugin is able to download "${imageUrl}"`);
            }
        },
        getMiddlewares: () => {
            const middlewares = getFnsFromPlugins('getMiddlewares');
            return (0, flatten_1.default)(middlewares.map((m) => m()));
        },
    };
};
exports.default = pluginLoader;
